--
-- Fendt380GTATurbo
-- Specialization for Fendt 380 GTA
--
-- @author  Manuel Leithner
-- @date  29/12/09
--

Fendt380GTATurbo = {};

function Fendt380GTATurbo.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function Fendt380GTATurbo:load(xmlFile)

	self.setFenderVisibility = SpecializationUtil.callSpecializationsFunction("setFenderVisibility");
	self.setHydraulicDirection = SpecializationUtil.callSpecializationsFunction("setHydraulicDirection");
	self.setLightState = SpecializationUtil.callSpecializationsFunction("setLightState");
	
	local hydraulicsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hydraulics#count"), 0);	
	self.hydraulics = {};	
	for i=1, hydraulicsCount do
		local hydraulicName = string.format("vehicle.hydraulics.hydraulic%d", i);		
		self.hydraulics[i] = {};		
		self.hydraulics[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#index"));
		self.hydraulics[i].punch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punch"));
		self.hydraulics[i].translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punchFixpoint"));
		self.hydraulics[i].fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#fixpoint"));
		self.hydraulics[i].scale = getXMLInt(xmlFile, hydraulicName .. "#scale");
		local ax, ay, az;		
		if self.hydraulics[i].punch ~= nil then
			ax, ay, az = getWorldTranslation(self.hydraulics[i].punch);
		else
			ax, ay, az = getWorldTranslation(self.hydraulics[i].node);
		end;
		if self.hydraulics[i].translationPunch ~= nil then
			local bx, by, bz = getWorldTranslation(self.hydraulics[i].translationPunch);		
			self.hydraulics[i].punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		end;
	end;
	self.setHydraulicTime = 30;

	-- Defines attributes of frontHydraulic
	self.frontHydraulic = {};
	self.frontHydraulic.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#hydraulic"));
	self.frontHydraulic.punch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#hydraulicPunch"));
	self.frontHydraulic.translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#hydraulicPunchFixPoint"));
	self.frontHydraulic.armFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#armFixPoint"));
	local ax, ay, az = getWorldTranslation(self.frontHydraulic.punch);
    local bx, by, bz = getWorldTranslation(self.frontHydraulic.translationPunch);
    self.frontHydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);	
	self.frontHydraulic.topArmPart2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#part2"));
	
	-- Defines attributes of backHydraulic
	self.backHydraulic = {};
	self.backHydraulic.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#hydraulic"));
	self.backHydraulic.punch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#hydraulicPunch"));
	self.backHydraulic.translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#hydraulicPunchFixPoint"));
	self.backHydraulic.rootArmFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArmFixPoint"));
	self.backHydraulic.rootArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArm"));
	self.backHydraulic.rootArmPart = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArmPart"));
	self.backHydraulic.armFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#armFixPoint"));
	self.backHydraulic.rootArmPartFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArmPartFixPoint"));
	local ax, ay, az = getWorldTranslation(self.backHydraulic.punch);
    local bx, by, bz = getWorldTranslation(self.backHydraulic.translationPunch);
    self.backHydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
	ax, ay, az = getWorldTranslation(self.backHydraulic.rootArmPart);
    bx, by, bz = getWorldTranslation(self.backHydraulic.rootArmPartFixPoint);
    self.backHydraulic.rootArmDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);

	
	self.backAttacherJoint = {};
	self.backAttacherJoint.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#back"));
	self.backAttacherJoint.upYPos = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.trailerAttacherJoints#upY"), 0);
	self.backAttacherJoint.downYPos = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.trailerAttacherJoints#downY"), 0);	
	local x,y,z = getTranslation(self.backAttacherJoint.node);
	self.backAttacherJoint.xPos = x;
	self.backAttacherJoint.zPos = z;	
	self.frontAttacherJoint = {};
	self.frontAttacherJoint.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#front"));
	setTranslation(self.backAttacherJoint.node, self.backAttacherJoint.xPos, self.backAttacherJoint.upYPos, self.backAttacherJoint.zPos);
	
	-- Defines attributes of drivingShaft
	self.drivingShaft = {};
	self.drivingShaft.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#index"));

	self.fenders = {};
	self.fenders.left = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fenders#index"));
	self.fenders.right = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fenders#index2"));
	
    local wiperAnimRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.wiper#rootNode"));
    self.wiperAnimCharSet = 0;
    if wiperAnimRootNode ~= nil and wiperAnimRootNode ~= 0 then
        self.wiperAnimCharSet = getAnimCharacterSet(wiperAnimRootNode);
        if self.wiperAnimCharSet ~= 0 then
            local clip = getAnimClipIndex(self.wiperAnimCharSet, getXMLString(xmlFile, "vehicle.wiper#clip"));
            assignAnimTrackClip(self.wiperAnimCharSet, 0, clip);
            setAnimTrackLoopState(self.wiperAnimCharSet, 0, true);
            local wiperAnimSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.wiper#speedScale"), 1);
			setAnimTrackSpeedScale(self.wiperAnimCharSet, 0, wiperAnimSpeedScale);
            self.wiperAnimDuration = getAnimClipDuration(self.wiperAnimCharSet, clip);
        end;
    end;
	self.isWiperActive = false;
	self.finishWiper = true;
	
	local numLights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.lightSources#count"), 0);
    self.lightsSource = {};
    for i=1, numLights do
        local lightnamei = string.format("vehicle.lightSources.source%d", i);
        self.lightsSource[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, lightnamei .. "#index"));
        setVisibility(self.lightsSource[i], false);
    end;
	
	self.moveDownIntervall = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.moveDownIntervall#ms"), 1000);
	self.currentLight = 0;	
	self.updateJoint = false;
	self.doLowering = false;
	self.loweringDone = false;
	self.nextAction = false;
	self.delay = 0;
end;

function Fendt380GTATurbo:delete()
end;

function Fendt380GTATurbo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Fendt380GTATurbo:keyEvent(unicode, sym, modifier, isDown)
	if sym == 308  and isDown then
		if table.getn(self.attachedImplements) == 2 then			
			self.loweringDone = false;
			self.delay = self.moveDownIntervall;
			self.doLowering = true;
			self.nextAction = not self.attacherJoints[1].moveDown;
			if self.attacherJoints[2].moveDown ~= not self.nextAction then 
				self.nextAction = false;
			end;
		end;
	end;
end;

function Fendt380GTATurbo:update(dt)

	if self:getIsActiveForInput() then
		-- Modulo division to get a controlled switch between each lights
        if InputBinding.hasEvent(InputBinding.TOGGLE_LIGHTS) then
			self:setLightState((self.currentLight + 1) % (table.getn(self.lightsSource) + 1));
        end;
	end;

	if self:getIsActive() then
	
		if self.doLowering then	
			local back = self.attacherJoints[1];
			local front = self.attacherJoints[2];	
			if not self.loweringDone then
				front.moveDown = self.nextAction;
				self.loweringDone = true;
			end;			
			if self.loweringDone then
				self.delay = self.delay - dt;				
				if self.delay <= 0 then
					back.moveDown = self.nextAction;
					self.doLowering = false;
				end;
			end;
		end;
			
		self.setHydraulicTime = 30;	

		if not self.finishWiper then
			if getAnimTrackTime(self.wiperAnimCharSet, 0) % self.wiperAnimDuration <= 100 then
				setAnimTrackTime(self.wiperAnimCharSet, 0, 0.0);
				disableAnimTrack(self.wiperAnimCharSet, 0);			
				self.finishWiper = true;
			end;			
		end;
		
		if g_currentMission.environment.lastRainScale > 0.1 and g_currentMission.environment.timeSinceLastRain < 30 then
			if not self.isWiperActive then
				enableAnimTrack(self.wiperAnimCharSet, 0);
				self.isWiperActive = true;
			end;
		else
			if self.isWiperActive then
				self.isWiperActive = false;
				self.finishWiper = false;
			end;
		end;
		
		-- oriented the hydraulic and the punch towards a fixpoint on the frontHydraulic
		if self.frontHydraulic ~= nil then
			local ax, ay, az = getWorldTranslation(self.frontHydraulic.node);
			local bx, by, bz = getWorldTranslation(self.frontHydraulic.armFixPoint);
			local x, y, z = worldDirectionToLocal(getParent(self.frontHydraulic.node), bx-ax, by-ay, bz-az);
			setDirection(self.frontHydraulic.node, x, y, z, 0, 1, 0);
			if self.frontHydraulic.punch ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				setTranslation(self.frontHydraulic.punch, 0, 0, distance-self.frontHydraulic.punchDistance);
			end;
		end;
		
		if self.drivingShaft ~= nil then
			local x,y,z = getRotation(self.wheels[1].driveNode);
			setRotation(self.drivingShaft.node, 0 ,0, x*2);	
		end;
		
		-- oriented all hydraulicparts towards the right place
		if self.backHydraulic ~= nil then
			-- set correct rotation on rootArm
			local x, y, z = getRotation(getParent(getParent(self.backHydraulic.armFixPoint)));
			setRotation(self.backHydraulic.rootArm, x, y, z);	

			-- oriented rootArmPart towards a fixpoint on the bottomArm
			local ax, ay, az = getWorldTranslation(self.backHydraulic.rootArmPart);
			local bx, by, bz = getWorldTranslation(self.backHydraulic.armFixPoint);
			x, y, z = worldDirectionToLocal(getParent(self.backHydraulic.rootArmPart), bx-ax, by-ay, bz-az);
			setDirection(self.backHydraulic.rootArmPart, x, y, z, 0, 0, 1);	
			
			
			-- scale the rootArmPart to get a better looking style (Do Not Copy This -> WORST implementation but "Yeah it works" ) :-)
			if self.backHydraulic.rootArmPart ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				local xScale, yScale, zScale = getScale(self.backHydraulic.rootArmPart);
				local newScale = zScale * (distance / self.backHydraulic.rootArmDistance);
				setScale(self.backHydraulic.rootArmPart, 1, 1, newScale);
			end;		
		
			-- oriented hydraulic towards a fixpoint on the rootArm
			ax, ay, az = getWorldTranslation(self.backHydraulic.node);
			bx, by, bz = getWorldTranslation(self.backHydraulic.rootArmFixPoint);
			x, y, z = worldDirectionToLocal(getParent(self.backHydraulic.node), bx-ax, by-ay, bz-az);
			setDirection(self.backHydraulic.node, x, y, z, 0, 1, 1);
			if self.backHydraulic.punch ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				local x,y,z = getTranslation(self.backHydraulic.punch);
				setTranslation(self.backHydraulic.punch, x, y, (distance-self.backHydraulic.punchDistance));
			end;			
		end;	
	end;
	
	if self.setHydraulicTime > 0 then
		for k,v in pairs(self.hydraulics) do 
			self:setHydraulicDirection(k);
		end;
		self.setHydraulicTime = self.setHydraulicTime - 1;
	end;
end;

function Fendt380GTATurbo:draw()	
end;

function Fendt380GTATurbo:attachImplement(implement)
	
	local jointType = implement.object.attacherJoint.jointType;
	local jointIndex = implement.jointDescIndex;
	
	if jointType == Vehicle.JOINTTYPE_IMPLEMENT then	
		-- index 1 defines back hydraulic
		if jointIndex == 1 then
			setVisibility(self.backAttacherJoint.node, false);
		elseif jointIndex == 2 then
			setVisibility(self.frontAttacherJoint.node, false);
			setVisibility(self.attacherJoints[jointIndex].topArm.rotationNode, true);
		end;
	elseif jointType == Vehicle.JOINTTYPE_TRAILER then
		if jointIndex == 3 then
			setVisibility(self.backAttacherJoint.node, true);	
			setTranslation(self.backAttacherJoint.node, self.backAttacherJoint.xPos, self.backAttacherJoint.upYPos, self.backAttacherJoint.zPos);	
		end;
	elseif jointType == Vehicle.JOINTTYPE_TRAILERLOW then
		if jointIndex == 5 then
			setVisibility(self.backAttacherJoint.node, true);	
			setTranslation(self.backAttacherJoint.node, self.backAttacherJoint.xPos, self.backAttacherJoint.downYPos, self.backAttacherJoint.zPos);
		end;
	end;
	self.updateJoint = true;
	
end;

function Fendt380GTATurbo:detachImplement(implementIndex)

	local implement = self.attachedImplements[implementIndex];
	local jointIndex = implement.jointDescIndex;

	if jointIndex == 1 then
		setVisibility(self.backAttacherJoint.node, true);
	elseif jointIndex == 2 then
		setVisibility(self.frontAttacherJoint.node, true);
		setVisibility(self.attacherJoints[jointIndex].topArm.rotationNode, false);
	elseif jointIndex == 5 then
		setTranslation(self.backAttacherJoint.node, self.backAttacherJoint.xPos, self.backAttacherJoint.upYPos, self.backAttacherJoint.zPos);
	end;
end;

function Fendt380GTATurbo:validateAttacherJoint(implement, jointDesc, dt)
    if self.updateJoint then
        self.updateJoint = false;
        return true;
    end;
    return false;
end;

function Fendt380GTATurbo:setFenderVisibility(isVisible)
	setVisibility(self.fenders.left, isVisible);
	setVisibility(self.fenders.right, isVisible);
end;

function Fendt380GTATurbo:setHydraulicDirection(index)
	local hydraulic = self.hydraulics[index];

	if hydraulic.fixPoint ~= nil then
		local ax, ay, az = getWorldTranslation(hydraulic.node);
		local bx, by, bz = getWorldTranslation(hydraulic.fixPoint);
		local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
		local scale = Utils.getNoNil(hydraulic.scale, 1);
		
		
		setDirection(hydraulic.node, x, y, z, 0, 1*scale, 0);
		if hydraulic.punch ~= nil then
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
			setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
		end;
	end;
end;

-- state 0: disabled
-- state 1: normal light
-- state 2: working light
function Fendt380GTATurbo:setLightState(state)
	if self.currentLight == state  then
		state = 0;
	end;
	if state >= 0 and state <= table.getn(self.lightsSource) then
		self:setLightsVisibility(state > 0);		
		if self.currentLight > 0 then
			setVisibility(self.lightsSource[self.currentLight], false);
		end;
		if state > 0 then
			setVisibility(self.lightsSource[state], true);
		end;
		self.currentLight = state;		
	else
		print("Error: given light state not available");
	end;
end;

function Fendt380GTATurbo:onLeave()
    if self.deactivateLightsOnLeave then
        self:setLightState(0);
    end;
	if self.isWiperActive then
		disableAnimTrack(self.wiperAnimCharSet, 0);	
	end;
end;

function Fendt380GTATurbo:onEnter()
	if self.isWiperActive then
		enableAnimTrack(self.wiperAnimCharSet, 0);	
	end;
end;